@@ -33,6 +33,9 @@
 #include "H5Alltypes.h"
 #include "H5private.h"		// for HDstrcpy
 
+#include <iostream>
+using namespace std;
+
 // There are a few comments that are common to most of the functions
 // defined in this file so they are listed here.
 // - getLocId is called by all functions, that call a C API, to get
@@ -130,9 +133,7 @@ Group CommonFG::openGroup( const char* name ) const
 
    // If the opening of the group failed, throw an exception
    if( group_id < 0 )
-   {
       throwException("openGroup", "H5Gopen2 failed");
-   }
 
    // No failure, create and return the Group object
    Group group( group_id );
@@ -174,9 +175,7 @@ DataSet CommonFG::createDataSet( const char* name, const DataType& data_type, co
 
    // If the creation of the dataset failed, throw an exception
    if( dataset_id < 0 )
-   {
       throwException("createDataSet", "H5Dcreate2 failed");
-   }
 
    // No failure, create and return the DataSet object
    DataSet dataset( dataset_id );
@@ -270,9 +269,8 @@ void CommonFG::link( H5L_type_t link_type, const char* curr_name, const char* ne
             break;
     } /* end switch */
 
-   if( ret_value < 0 ) {
+   if( ret_value < 0 )
       throwException("link", "creating link failed");
-   }
 }
 
 //--------------------------------------------------------------------------
@@ -300,9 +298,7 @@ void CommonFG::unlink( const char* name ) const
 {
    herr_t ret_value = H5Ldelete( getLocId(), name, H5P_DEFAULT );
    if( ret_value < 0 )
-   {
       throwException("unlink", "H5Ldelete failed");
-   }
 }
 
 //--------------------------------------------------------------------------
@@ -336,9 +332,7 @@ void CommonFG::move( const char* src, const char* dst ) const
 {
    herr_t ret_value = H5Lmove( getLocId(), src, H5L_SAME_LOC, dst, H5P_DEFAULT, H5P_DEFAULT );
    if( ret_value < 0 )
-   {
       throwException("move", "H5Lmove failed");
-   }
 }
 
 //--------------------------------------------------------------------------
@@ -371,9 +365,7 @@ void CommonFG::getObjinfo( const char* name, hbool_t follow_link, H5G_stat_t& st
 {
    herr_t ret_value = H5Gget_objinfo( getLocId(), name, follow_link, &statbuf );
    if( ret_value < 0 )
-   {
       throwException("getObjinfo", "H5Gget_objinfo failed");
-   }
 }
 
 //--------------------------------------------------------------------------
@@ -400,9 +392,7 @@ void CommonFG::getObjinfo( const char* name, H5G_stat_t& statbuf ) const
 {
    herr_t ret_value = H5Gget_objinfo( getLocId(), name, 0, &statbuf );
    if( ret_value < 0 )
-   {
       throwException("getObjinfo", "H5Gget_objinfo failed");
-   }
 }
 
 //--------------------------------------------------------------------------
@@ -440,9 +430,8 @@ H5std_string CommonFG::getLinkval( const char* name, size_t size ) const
     {
 	ret_value = H5Lget_info(getLocId(), name, &linkinfo, H5P_DEFAULT);
 	if( ret_value < 0 )
-	{
 	    throwException("getLinkval", "H5Lget_info to find buffer size failed");
-	}
+
 	val_size = linkinfo.u.val_size;
     }
 
@@ -450,12 +439,15 @@ H5std_string CommonFG::getLinkval( const char* name, size_t size ) const
     if (val_size > 0)
     {
 	value_C = new char[val_size+1];  // temporary C-string for C API
+	HDmemset(value_C, 0, val_size+1); // clear buffer
 
 	ret_value = H5Lget_val(getLocId(), name, value_C, val_size, H5P_DEFAULT);
 	if( ret_value < 0 )
 	{
+	    delete []value_C;
 	    throwException("getLinkval", "H5Lget_val failed");
 	}
+
 	value = H5std_string(value_C);
 	delete []value_C;
     }
@@ -474,140 +466,6 @@ H5std_string CommonFG::getLinkval( const H5std_string& name, size_t size ) const
    return( getLinkval( name.c_str(), size ));
 }
 
-//--------------------------------------------------------------------------
-// Function:	CommonFG::setComment
-///\brief	Sets or resets the comment for an object specified by its name.
-///\param	name  - IN: Name of the object
-///\param	comment - IN: New comment
-///\exception	H5::FileIException or H5::GroupIException
-///\par	Description
-///		If \a comment is an empty string or a null pointer, the comment
-///		message is removed from the object.
-///		Comments should be relatively short, null-terminated, ASCII
-///		strings.  They can be attached to any object that has an
-///		object header, e.g., data sets, groups, named data types,
-///		and data spaces, but not symbolic links.
-// Programmer	Binh-Minh Ribler - 2000
-// Modification
-//	2007: QAK modified to use H5O APIs; however the first parameter is
-//		no longer just file or group, this function should be moved
-//		to another class to accommodate attribute, dataset, and named
-//		datatype. - BMR
-//--------------------------------------------------------------------------
-void CommonFG::setComment( const char* name, const char* comment ) const
-{
-   herr_t ret_value = H5Oset_comment_by_name( getLocId(), name, comment, H5P_DEFAULT );
-   if( ret_value < 0 )
-   {
-      throwException("setComment", "H5Oset_comment_by_name failed");
-   }
-}
-
-//--------------------------------------------------------------------------
-// Function:	CommonFG::setComment
-///\brief	This is an overloaded member function, provided for convenience.
-///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name and \a comment.
-// Programmer	Binh-Minh Ribler - 2000
-//--------------------------------------------------------------------------
-void CommonFG::setComment( const H5std_string& name, const H5std_string& comment ) const
-{
-   setComment( name.c_str(), comment.c_str() );
-}
-
-//--------------------------------------------------------------------------
-// Function:	CommonFG::removeComment
-///\brief	Removes the comment from an object specified by its name.
-///\param	name  - IN: Name of the object
-///\exception	H5::FileIException or H5::GroupIException
-// Programmer	Binh-Minh Ribler - May 2005
-//	2007: QAK modified to use H5O APIs; however the first parameter is
-//		no longer just file or group, this function should be moved
-//		to another class to accommodate attribute, dataset, and named
-//		datatype. - BMR
-//--------------------------------------------------------------------------
-void CommonFG::removeComment(const char* name) const
-{
-   herr_t ret_value = H5Oset_comment_by_name(getLocId(), name, NULL, H5P_DEFAULT);
-   if( ret_value < 0 )
-   {
-      throwException("removeComment", "H5Oset_comment_by_name failed");
-   }
-}
-
-//--------------------------------------------------------------------------
-// Function:	CommonFG::removeComment
-///\brief	This is an overloaded member function, provided for convenience.
-///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
-// Programmer	Binh-Minh Ribler - May 2005
-//--------------------------------------------------------------------------
-void CommonFG::removeComment(const H5std_string& name) const
-{
-   removeComment (name.c_str());
-}
-
-//--------------------------------------------------------------------------
-// Function:	CommonFG::getComment
-///\brief	Retrieves comment for the specified object and its comment's
-///		length.
-///\param	name  - IN: Name of the object
-///\param	bufsize - IN: Length of the comment to retrieve
-///\return	Comment string
-///\exception	H5::FileIException or H5::GroupIException
-// Programmer	Binh-Minh Ribler - 2000
-//	2007: QAK modified to use H5O APIs; however the first parameter is
-//		no longer just file or group, this function should be moved
-//		to another class to accommodate attribute, dataset, and named
-//		datatype. - BMR
-//--------------------------------------------------------------------------
-H5std_string CommonFG::getComment( const char* name, size_t bufsize ) const
-{
-   // bufsize is default to 256
-   // temporary variable
-   hid_t loc_id = getLocId();   // temporary variable
-
-   // temporary C-string for the object's comment; bufsize already including
-   // null character
-   char* comment_C = new char[bufsize];
-   ssize_t ret_value = H5Oget_comment_by_name(loc_id, name, comment_C, bufsize, H5P_DEFAULT);
-
-   // if the actual length of the comment is longer than bufsize and bufsize
-   // was the default value, i.e., not given by the user, then call
-   // H5Oget_comment_by_name again with the correct value.
-   // If the call to H5Oget_comment_by_name returned an error, skip this block
-   // and throw an exception below.
-   if (ret_value >= 0 && (size_t)ret_value > bufsize && bufsize == 256)
-   {
-	size_t new_size = ret_value;
-	delete []comment_C;
-	comment_C = new char[new_size];	// new_size including null terminator
-	ret_value = H5Oget_comment_by_name(loc_id, name, comment_C, new_size, H5P_DEFAULT);
-   }
-
-   // if H5Oget_comment_by_name returns SUCCEED, return the string comment,
-   // otherwise, throw an exception
-   if( ret_value < 0 )
-   {
-      throwException("getComment", "H5Oget_comment_by_name failed");
-   }
-   H5std_string comment = H5std_string(comment_C);
-   delete []comment_C;
-   return (comment);
-}
-
-//--------------------------------------------------------------------------
-// Function:	CommonFG::getComment
-///\brief	This is an overloaded member function, provided for convenience.
-///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
-// Programmer	Binh-Minh Ribler - 2000
-//--------------------------------------------------------------------------
-H5std_string CommonFG::getComment( const H5std_string& name, size_t bufsize ) const
-{
-   return( getComment( name.c_str(), bufsize ));
-}
-
 //--------------------------------------------------------------------------
 // Function:	CommonFG::mount
 ///\brief	Mounts the file \a child onto this group.
@@ -615,9 +473,9 @@ H5std_string CommonFG::getComment( const H5std_string& name, size_t bufsize ) co
 ///\param	child - IN: File to mount
 ///\param	plist - IN: Property list to use
 ///\exception	H5::FileIException or H5::GroupIException
-// Programmer	Binh-Minh Ribler - 2000
+// Programmer	Binh-Minh Ribler - 2014 (original 2000)
 //--------------------------------------------------------------------------
-void CommonFG::mount( const char* name, H5File& child, PropList& plist ) const
+void CommonFG::mount(const char* name, const H5File& child, const PropList& plist ) const
 {
    // Obtain identifiers for C API
    hid_t plist_id = plist.getId();
@@ -628,21 +486,46 @@ void CommonFG::mount( const char* name, H5File& child, PropList& plist ) const
 
    // Raise exception if H5Fmount returns negative value
    if( ret_value < 0 )
-   {
       throwException("mount", "H5Fmount failed");
-   }
+}
+
+//--------------------------------------------------------------------------
+// Function:	CommonFG::mount
+///\brief	This is an overloaded member function, kept for backward
+///		compatibility.  It differs from the above function in that it
+///		misses const's.  This wrapper will be removed in future release.
+///\param	name  - IN: Name of the group
+///\param	child - IN: File to mount
+///\param	plist - IN: Property list to use
+///\exception	H5::FileIException or H5::GroupIException
+// Programmer	Binh-Minh Ribler - 2000
+//--------------------------------------------------------------------------
+void CommonFG::mount(const char* name, H5File& child, PropList& plist) const
+{
+   mount(name, (const H5File)child, (const PropList)plist);
 }
 
 //--------------------------------------------------------------------------
 // Function:	CommonFG::mount
 ///\brief	This is an overloaded member function, provided for convenience.
-///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		It takes an \c H5std_string for \a name.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
-void CommonFG::mount( const H5std_string& name, H5File& child, PropList& plist ) const
+void CommonFG::mount(const H5std_string& name, const H5File& child, const PropList& plist) const
 {
-   mount( name.c_str(), child, plist );
+   mount(name.c_str(), child, plist);
+}
+
+//--------------------------------------------------------------------------
+// Function:	CommonFG::mount
+///\brief	This is an overloaded member function, kept for backward
+///		compatibility.  It differs from the above function in that it
+///		misses const's.  This wrapper will be removed in future release.
+// Programmer	Binh-Minh Ribler - 2014
+//--------------------------------------------------------------------------
+void CommonFG::mount(const H5std_string& name, H5File& child, PropList& plist) const
+{
+   mount(name.c_str(), (const H5File)child, (const PropList)plist);
 }
 
 //--------------------------------------------------------------------------
@@ -659,9 +542,7 @@ void CommonFG::unmount( const char* name ) const
 
    // Raise exception if H5Funmount returns negative value
    if( ret_value < 0 )
-   {
       throwException("unmount", "H5Funmount failed");
-   }
 }
 
 //--------------------------------------------------------------------------
@@ -1006,9 +887,7 @@ hsize_t CommonFG::getNumObjs() const
 
    herr_t ret_value = H5Gget_info(getLocId(), &ginfo);
    if(ret_value < 0)
-   {
       throwException("getNumObjs", "H5Gget_info failed");
-   }
    return (ginfo.nlinks);
 }
 
@@ -1032,14 +911,20 @@ H5std_string CommonFG::getObjnameByIdx(hsize_t idx) const
     // call H5Lget_name_by_idx with name as NULL to get its length
     ssize_t name_len = H5Lget_name_by_idx(getLocId(), ".", H5_INDEX_NAME, H5_ITER_INC, idx, NULL, 0, H5P_DEFAULT);
     if(name_len < 0)
-    {
       throwException("getObjnameByIdx", "H5Lget_name_by_idx failed");
-    }
 
     // now, allocate C buffer to get the name
     char* name_C = new char[name_len+1];
+    HDmemset(name_C, 0, name_len+1); // clear buffer
+
     name_len = H5Lget_name_by_idx(getLocId(), ".", H5_INDEX_NAME, H5_ITER_INC, idx, name_C, name_len+1, H5P_DEFAULT);
 
+    if (name_len < 0)
+    {
+	delete []name_C;
+	throwException("getObjnameByIdx", "H5Lget_name_by_idx failed");
+    }
+
     // clean up and return the string
     H5std_string name = H5std_string(name_C);
     delete []name_C;
@@ -1067,9 +952,8 @@ ssize_t CommonFG::getObjnameByIdx(hsize_t idx, char* name, size_t size) const
 {
    ssize_t name_len = H5Lget_name_by_idx(getLocId(), ".", H5_INDEX_NAME, H5_ITER_INC, idx, name, size, H5P_DEFAULT);
    if(name_len < 0)
-   {
       throwException("getObjnameByIdx", "H5Lget_name_by_idx failed");
-   }
+
    return (name_len);
 }
 
@@ -1082,18 +966,142 @@ ssize_t CommonFG::getObjnameByIdx(hsize_t idx, char* name, size_t size) const
 //--------------------------------------------------------------------------
 ssize_t CommonFG::getObjnameByIdx(hsize_t idx, H5std_string& name, size_t size) const
 {
-   char* name_C = new char[size];
-   ssize_t name_len = getObjnameByIdx(idx, name_C, size);
+   char* name_C = new char[size+1]; // temporary C-string for object name
+   HDmemset(name_C, 0, size+1); // clear buffer
+
+   // call overloaded function to get the name
+   ssize_t name_len = getObjnameByIdx(idx, name_C, size+1);
    if(name_len < 0)
    {
+      delete []name_C;
       throwException("getObjnameByIdx", "H5Lget_name_by_idx failed");
    }
+
+   // clean up and return the string
    name = H5std_string(name_C);
    delete []name_C;
    return (name_len);
 }
 
+//--------------------------------------------------------------------------
+// Function:	CommonFG::childObjType
+///\brief	Returns the type of an object in this file/group, given the
+///		object's name.
+///\param	objname - IN: Name of the object
+///\return	Object type, which can have the following values for group,
+///		dataset, and named datatype
+///		\li \c H5O_TYPE_GROUP
+///		\li \c H5O_TYPE_DATASET
+///		\li \c H5O_TYPE_NAMED_DATATYPE
+///		Refer to the C API documentation for more details:
+///		http://www.hdfgroup.org/HDF5/doc/RM/RM_H5O.html#Object-GetInfo
+///\exception	H5::FileIException or H5::GroupIException
+///		Exception will be thrown when:
+///		- an error returned by the C API
+///		- object type is not one of the valid values above
+// Programmer	Binh-Minh Ribler - April, 2014
+//--------------------------------------------------------------------------
+H5O_type_t CommonFG::childObjType(const char* objname) const
+{
+    H5O_info_t objinfo;
+    H5O_type_t objtype = H5O_TYPE_UNKNOWN;
+
+    // Use C API to get information of the object
+    herr_t ret_value = H5Oget_info_by_name(getLocId(), objname, &objinfo, H5P_DEFAULT);
+
+    // Throw exception if C API returns failure
+    if (ret_value < 0)
+	throwException("childObjType", "H5Oget_info_by_name failed");
+    // Return a valid type or throw an exception for unknown type
+    else
+      switch (objinfo.type)
+      {
+	case H5O_TYPE_GROUP:
+	case H5O_TYPE_DATASET:
+	case H5O_TYPE_NAMED_DATATYPE:
+	    objtype = objinfo.type;
+	    break;
+	default:
+	    throwException("childObjType", "Unknown type of object");
+      }
+    return(objtype);
+}
+
+//--------------------------------------------------------------------------
+// Function:	CommonFG::childObjType
+///\brief	This is an overloaded member function, provided for convenience.
+///		It takes an \a H5std_string for the object's name.
+///\brief	Returns the type of an object in this group, given the
+///		object's name.
+///\param	objname - IN: Name of the object (H5std_string&)
+///\exception	H5::FileIException or H5::GroupIException
+// Programmer	Binh-Minh Ribler - April, 2014
+//--------------------------------------------------------------------------
+H5O_type_t CommonFG::childObjType(const H5std_string& objname) const
+{
+    // Use overloaded function
+    H5O_type_t objtype = childObjType(objname.c_str());
+    return(objtype);
+}
+
+//--------------------------------------------------------------------------
+// Function:	CommonFG::childObjType
+///\brief	Returns the type of an object in this file/group, given the
+///		object's index and its type and order.
+///\param	index - IN: Position of the object
+///\param	index_type - IN: Type of the index, default to H5_INDEX_NAME
+///\param	order - IN: Traversing order, default to H5_ITER_INC
+///\param	objname - IN: Name of the object, default to "."
+///\return	Object type, which can have the following values for group,
+///		dataset, and named datatype
+///		\li \c H5O_TYPE_GROUP
+///		\li \c H5O_TYPE_DATASET
+///		\li \c H5O_TYPE_NAMED_DATATYPE
+///		Refer to the C API documentation for more details:
+///		http://www.hdfgroup.org/HDF5/doc/RM/RM_H5O.html#Object-GetInfo
+///\exception	H5::FileIException or H5::GroupIException
+///		Exception will be thrown when:
+///		- an error returned by the C API
+///		- object type is not one of the valid values above
+// Developer's Notes:
+//	- this overload uses H5Oget_info_by_idx instead of H5Oget_info_by_name
+//	  like the previous childObjType()
+//	- index is the required argument so, first
+//	- objname is last because it's more likely the location is already
+//	  fully specified
+//	- Leave property list out for now because C API is not using it, it
+//	  can be added later when needed.
+// Programmer	Binh-Minh Ribler - April, 2014
+//--------------------------------------------------------------------------
+H5O_type_t CommonFG::childObjType(hsize_t index, H5_index_t index_type, H5_iter_order_t order, const char* objname) const
+{
+    herr_t ret_value;
+    H5O_info_t objinfo;
+    H5O_type_t objtype = H5O_TYPE_UNKNOWN;
+
+    // Use C API to get information of the object
+    ret_value = H5Oget_info_by_idx(getLocId(), objname, index_type, order, index, &objinfo, H5P_DEFAULT);
+
+    // Throw exception if C API returns failure
+    if (ret_value < 0)
+	throwException("childObjType", "H5Oget_info_by_idx failed");
+    // Return a valid type or throw an exception for unknown type
+    else
+      switch (objinfo.type)
+      {
+	case H5O_TYPE_GROUP:
+	case H5O_TYPE_DATASET:
+	case H5O_TYPE_NAMED_DATATYPE:
+	    objtype = objinfo.type;
+	    break;
+	default:
+	    throwException("childObjType", "Unknown type of object");
+      }
+    return(objtype);
+}
+
 #ifndef H5_NO_DEPRECATED_SYMBOLS
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
 //--------------------------------------------------------------------------
 // Function:	CommonFG::getObjTypeByIdx
 ///\brief	Returns the type of an object in this group, given the
@@ -1107,9 +1115,8 @@ H5G_obj_t CommonFG::getObjTypeByIdx(hsize_t idx) const
 {
    H5G_obj_t obj_type = H5Gget_objtype_by_idx(getLocId(), idx);
    if (obj_type == H5G_UNKNOWN)
-   {
       throwException("getObjTypeByIdx", "H5Gget_objtype_by_idx failed");
-   }
+
    return (obj_type);
 }
 
@@ -1119,7 +1126,7 @@ H5G_obj_t CommonFG::getObjTypeByIdx(hsize_t idx) const
 ///		It differs from the above function because it also provides
 ///		the returned object type in text (char*)
 ///\param	idx       - IN: Transient index of the object
-///\param	type_name - IN: Object type in text
+///\param	type_name - OUT: Object type in text
 ///\return	Object type
 ///\exception	H5::FileIException or H5::GroupIException
 // Programmer	Binh-Minh Ribler - May, 2010
@@ -1135,9 +1142,7 @@ H5G_obj_t CommonFG::getObjTypeByIdx(hsize_t idx, char* type_name) const
 	case H5G_TYPE: HDstrcpy(type_name, "datatype"); break;
 	case H5G_UNKNOWN:
 	default:
-   	{
 	   throwException("getObjTypeByIdx", "H5Gget_objtype_by_idx failed");
-	}
    }
    return (obj_type);
 }
@@ -1147,7 +1152,7 @@ H5G_obj_t CommonFG::getObjTypeByIdx(hsize_t idx, char* type_name) const
 ///		It differs from the above function because it also provides
 ///		the returned object type in text (H5std_string&)
 ///\param	idx       - IN: Transient index of the object
-///\param	type_name - IN: Object type in text
+///\param	type_name - OUT: Object type in text
 ///\return	Object type
 ///\exception	H5::FileIException or H5::GroupIException
 // Programmer	Binh-Minh Ribler - January, 2003
@@ -1163,14 +1168,14 @@ H5G_obj_t CommonFG::getObjTypeByIdx(hsize_t idx, H5std_string& type_name) const
 	case H5G_TYPE: type_name = H5std_string("datatype"); break;
 	case H5G_UNKNOWN:
 	default:
-   	{
 	   throwException("getObjTypeByIdx", "H5Gget_objtype_by_idx failed");
-	}
    }
    return (obj_type);
 }
+#endif // DOXYGEN_SHOULD_SKIP_THIS
 #endif /* H5_NO_DEPRECATED_SYMBOLS */
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
 //--------------------------------------------------------------------------
 // Function:	CommonFG default constructor
 ///\brief	Default constructor.
@@ -1184,6 +1189,7 @@ CommonFG::CommonFG() {}
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 CommonFG::~CommonFG() {}
+#endif // DOXYGEN_SHOULD_SKIP_THIS
 
 #ifndef H5_NO_NAMESPACE
 }
